#include <Servo.h> 
#include "EasingLibrary.h"
#include <NewPing.h>
#include "Wire.h"

// INPUTS: A3, A2, A1, A0, 2 (FR. TOP)
// GND
#define LeftFoot  A0  // analog pin 0 used for edge detection
#define RightFoot  A1  // analog pin 1 used for edge detection
// A2 used for random seed generation
#define AudioPin  A3  // analog pin 3 used for audio detection
#define TRIGGER_PIN  4  // Arduino pin tied to trigger pin on the ultrasonic sensor.
#define ECHO_PIN     3  // Arduino pin tied to echo pin on the ultrasonic sensor.
// Vcc
#define MAX_DISTANCE 15 // Maximum distance we want to ping for (in centimeters). Maximum sensor distance is rated at 400-500cm.

NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE); // NewPing setup of pins and maximum distance.

SineEase Ease;

Servo RightHipServo;  // setup servo objects 
Servo LeftHipServo;
Servo RightAnkleServo;
Servo LeftAnkleServo;

const int DistanceDance = 250;  // activates dance mode at power up

int Right = 1;
int Left = 2;
int Foot;  // for determining turning patterns
int CalibrationCounter = 0;  // dictates which part of the calibration cycle should be used
int LightDetected;  // holds edge sensor value
int EdgeDetection;  // used to setup position during object avoidance
int DarkCalibration;  // endge sensor voltage when foot is down.
int Numerator = 5;  // numerator of calibration correction factor
int Denominator = 12;  // denominator of the calibration correction factor;
int LightThreshold;  // lowest ADC value from CdS cells to pass edge detection
int StepDelay = 1000;  // pause between servo commands in uS; controls walking speed. 250 = very fast 2500 = good for testing 1500 = good walk
int Distance;  // ultrasonic pulse travel time in uS
int DistanceMax = 500;  // maximum ultrasonic flight time to trigger obstacle avoidance
int AudioFlag = 0;  // 1 = clap detected
int DanceTrigger = 0;  // used to trigger dance mode loop
int RandomNumber;
int Duration = 199;  // time units in easing function, zero indexed
double easedPosition;  // adjustment value to servo between steps
int i; // counter used in for/next loops
int Pos;  // servo position in microseconds
int HipRange = 160;  // range in uS between servo travel endpoints
int AnkleRange = 170;
int RightHipMid = 1350;
int RightHipLeft = RightHipMid - HipRange;  // servo positions in uS. left refers to the direction the body turns in relation to the leg 
int RightHipRight = RightHipMid + HipRange;
int LeftHipMid = 1470;
int LeftHipLeft = LeftHipMid - HipRange;  
int LeftHipRight = LeftHipMid + HipRange;
int RightAnkleMid = 1580;  //1343
int RightAnkleUp = RightAnkleMid - AnkleRange;  // up refers to the outer edge of the foot that is being raised
int RightAnkleDown = RightAnkleMid + AnkleRange;
int LeftAnkleMid = 1540;  //1550
int LeftAnkleUp = LeftAnkleMid + AnkleRange;
int LeftAnkleDown = LeftAnkleMid - AnkleRange;


void setup() 
{ 
  // top output header = 12
  RightHipServo.attach(5);  // set up pins for servos
  LeftHipServo.attach(6);
  RightAnkleServo.attach(7);
  LeftAnkleServo.attach(8);

  randomSeed(analogRead(2));  // start random number generator using value off floating analog pin 2
  pinMode(RightFoot,INPUT);  // right foot sensor
  pinMode(LeftFoot,INPUT);
  pinMode(AudioPin, INPUT);  // clap detector

//  Serial.begin(9600);  // setup for terminal debug
  Wire.begin(); // join i2c bus (address optional for master)
// Notes:
// Modes: 0= ack(flash), 1=off, 2=walking, 3=fanmode, 4&5=avoidance, 6=static

  RightHipServo.writeMicroseconds(RightHipMid);  // center all servos
  LeftHipServo.writeMicroseconds(LeftHipMid);
  RightAnkleServo.writeMicroseconds(RightAnkleMid);
  LeftAnkleServo.writeMicroseconds(LeftAnkleMid);

  Ease.setDuration(Duration);  //  sets overall time units for easing function

  delay(5000);  // pause before the big show

  GetDistance();  //check for hand in front of bot
  if (Distance < DistanceDance && Distance > 0) {  // detected. 0 returned if sensor unplugged. argument allows testing without sensor
  SendI2C(0);  // flash ack
  DanceTrigger = 1;  // sets flag to go to dance mode during startup
  }
  if (DanceTrigger == 1) {  // check for flag
    delay(1000);  // slight delay
    SendI2C(6);  // start static LED sequences
    int PinCheck;
SecondLoop:
    Listen();
    if(AudioFlag == 1) {
      Celebrate();
      AudioFlag = 0;
    }
    goto SecondLoop;  // dance mode loop goto
  }

  SendI2C(2);  // start walking LED sequence
  Foot = Left;  // flags left foot as moving
  Calibrate();  // sets value for what a foot on solid ground looks like
  LeanRight();    // first movement from startup; called during startup because it's not used in normal walking loop
}


void loop() {  // main loop
  // starts in base position, left foot in the air
  Navigate();
  LeftFootStepForwardSecondHalf();  // begin walk forward loop
  Calibrate();  // sets value for what a foot over open air looks like
  LeftFootPutDown();  // sensor will be looking right
  delay(30);
  TestFooting();  // compares reading from edge detector to threshold value
  Listen();  // listens for clapping, sets flag if heard
  
  Foot = Right;  // right foot in motion
  Calibrate();  // sets foot down value
  RightFootStepForwardFirstHalf();  // facing forward, one leg in the air, hips straight
  Listen();  // listens for clapping and sets flag if heard
  HearingCheck();  // checks for clapping flag and handles celebratory dancing that ensues
  Navigate();  // ultrasonic sensor check, go-no go decisions, and maneuvering sequences
  RightFootStepForwardSecondHalf();  // moves leg forward
  Calibrate();  // foot up
  RightFootPutDown();  //sensor looking left
  delay(30);
  TestFooting();
  Listen();
  Foot = Left;
  Calibrate();  // foot down
  LeftFootStepForwardFirstHalf();  //lt leg up
  Listen();
  HearingCheck();
}


//                                                        end loops
/*************************************************************************************************************************
 * Movement Sequences                                                 *
 **************************************************************************************************************************/

// movement sequences consist of one or many Individual Moves

void LeanRight() {  // startup move
  for( i=0;i<=Duration;i++)    {
    AnkleMove1(Left);    
    AnkleMove2 (Right);
    delayMicroseconds (StepDelay);
  }
}
void LeanLeft() {
  for( i=0;i<=Duration;i++)    {
    AnkleMove1(Right);  
    AnkleMove2 (Left);    
    delayMicroseconds (StepDelay);
  }
}
void RightFootStepForwardFirstHalf() {
  for( i=0;i<=Duration;i++)    {
    AnkleMove1(Right);    
    AnkleMove2 (Left);    
    HipMove2i (Right);
    HipMove2i (Left);    
    delayMicroseconds (StepDelay);
  }
  Foot = Right;  // referenced during turning routine
}
void LeftFootStepForwardFirstHalf() {
  for( i=0;i<=Duration;i++)    {
    AnkleMove1(Left);    
    AnkleMove2 (Right);    
    HipMove4i (Right);    
    HipMove4i (Left);    
    delayMicroseconds (StepDelay);
  }
  Foot = Left;  // referenced during turning routine
}
void RightFootStepForwardSecondHalf() {
  for( i=0;i<=Duration;i++){
    HipMove3o (Left);    
    HipMove3o (Right);    
    delayMicroseconds (StepDelay);
  }
}
void LeftFootStepForwardSecondHalf() {
  for( i=0;i<=Duration;i++) {
    HipMove1o (Right);    
    HipMove1o (Left);    
    delayMicroseconds (StepDelay);
  }
}
void RightFootPutDown() {
  for( i=0;i<=Duration;i++)    {
    AnkleMove3i (Right);
    AnkleMove4o (Left);    
    delayMicroseconds (StepDelay);
  }
}
void LeftFootPutDown() {
  for( i=0;i<=Duration;i++)    {
    AnkleMove4o (Right);   
    AnkleMove3i (Left);   
    delayMicroseconds (StepDelay);
  }
}
void BeginTurnRight() { // make following beginturnright
  for( i=0;i<=Duration;i++)    {
    HipMove3i(Right);
    delayMicroseconds (StepDelay);
  }
}      
void BeginTurnLeft() {  // make following beginturnleft
  for( i=0;i<=Duration;i++)    {
    HipMove1i(Left);
    delayMicroseconds(StepDelay);
  }
}
void PivotLeft() {  // make following pivotleft
  LeanLeft();
  for( i=0;i<=Duration;i++)    {
    HipMove2i(Left);    
    delayMicroseconds(StepDelay);  
  }
  RightFootPutDown();
}
void PivotRight() {  // make following pivotright
  LeanRight();
  for( i=0;i<=Duration;i++)    {
    HipMove4i(Right);     
    delayMicroseconds(StepDelay);  
  }
  LeftFootPutDown();
}
void SpinRight() {
  LeanLeft();
  BeginTurnRight();
  RightFootPutDown();
  PivotRight();
  LeanLeft();
  for( i=0;i<=Duration;i++) {
    HipMove1(Left);
    delayMicroseconds(StepDelay);
  }
  RightFootPutDown();
  LeanRight();
  for( i=0;i<=Duration;i++) {
    HipMove2(Left);
    delayMicroseconds(StepDelay);
  }
  LeftFootPutDown();  
}
void SpinLeft() {
  LeanRight();
  BeginTurnLeft();
  LeftFootPutDown();
  PivotLeft();
  LeanRight();
  for( i=0;i<=Duration;i++) {
    HipMove3(Right);
    delayMicroseconds(StepDelay);
  }
  LeftFootPutDown();
  LeanLeft();
  for( i=0;i<=Duration;i++) {
    HipMove4(Right);
    delayMicroseconds(StepDelay);
  }
  RightFootPutDown();  
}
void TurnLeft() {
  LeanRight();
  BeginTurnLeft();
  LeftFootPutDown();
  PivotLeft();
}
void TurnRight() {
  LeanLeft();
  BeginTurnRight();
  RightFootPutDown();
  PivotRight();
}
void RightFootStepBackward() {
  RightFootStepBackwardFirstHalf();
  RightFootStepBackwardSecondHalf();
}
void RightFootStepBackwardFirstHalf() {
  for( i=0;i<=Duration;i++) {
    HipMove4i(Right);
    HipMove4i(Left);
    delayMicroseconds(StepDelay);
  } 
}  
void RightFootStepBackwardSecondHalf() {
  for( i=0;i<=Duration;i++) {
    HipMove1o(Right);
    HipMove1o(Left);
    delayMicroseconds(StepDelay);
  }
}
void LeftFootStepBackward() {
  LeftFootStepBackwardFirstHalf();
  LeftFootStepBackwardSecondHalf();
}
void LeftFootStepBackwardFirstHalf() {
  for( i=0;i<=Duration;i++) {
    HipMove2i(Right);
    HipMove2i(Left);
    delayMicroseconds(StepDelay);
  } 
}  
void LeftFootStepBackwardSecondHalf() {
  for( i=0;i<=Duration;i++) {
    HipMove3o(Right);
    HipMove3o(Left);
    delayMicroseconds(StepDelay);
  }
}
void RightAnkleWiggle() {
  for (int j=0;j<3;j++) {
    for( i=0;i<=Duration;i++) {
      AnkleMove3i(Right);
      delayMicroseconds(StepDelay);
    }
    for( i=0;i<=Duration;i++) {
      AnkleMove2i(Right);
      delayMicroseconds(StepDelay);
    }
    for( i=0;i<=Duration;i++) {
      AnkleMove4i(Right);
      delayMicroseconds(StepDelay);
    }
    for( i=0;i<=Duration;i++) {
      AnkleMove1i(Right);
      delayMicroseconds(StepDelay);
    }
  }  //  x2
}
void LeftAnkleWiggle() {
  for (int j=0;j<3;j++) {
    for ( i=0;i<=Duration;i++) {
      AnkleMove3i(Left);
      delayMicroseconds(StepDelay);
    }
    for ( i=0;i<=Duration;i++) {
      AnkleMove2i(Left);
      delayMicroseconds(StepDelay);
    }
    for ( i=0;i<=Duration;i++) {
      AnkleMove4i(Left);
      delayMicroseconds(StepDelay);
    }
    for ( i=0;i<=Duration;i++) {
      AnkleMove1i(Left);
      delayMicroseconds(StepDelay);
    }
  }
}
void RightLegTwist() {
  for( i=0;i<=Duration;i++) {
    HipMove3i(Right);
    AnkleMove3i(Right);
    delayMicroseconds(StepDelay);
  }
  for( i=0;i<=Duration;i++) {
    HipMove4i(Right);
    AnkleMove1i(Right);
    delayMicroseconds(StepDelay);
  }
}
void LeftLegTwist() {
  for( i=0;i<=Duration;i++) {
    HipMove1i(Left);
    AnkleMove3i(Left);
    delayMicroseconds(StepDelay);
  }
  for ( i=0;i<=Duration;i++) {
    HipMove2i(Left);
    AnkleMove1i(Left);
    delayMicroseconds(StepDelay);
  }
}
//                                    end Movement Sequences
/***********************************************************************************************
 * Dance Moves                                            *
 ***********************************************************************************************/
void ShuffleStep() {
  LeanRight();    // first movement from startup; not used in normal walking routine
  //------
  LeftFootStepForwardSecondHalf();  // begin walk forward loop
  LeftFootPutDown();  //sensor looking right
  RightFootStepForwardFirstHalf();  // base refers to base position; facing forward, one leg in the air, hips straight; this cycle it's rt leg up
  RightFootStepForwardSecondHalf();
  RightFootStepBackward();
  RightFootPutDown();
  LeanRight();
  LeftFootStepBackward();
  for( i=0;i<=Duration;i++) {
    HipMove4i(Right);
    HipMove4i(Left);
    delayMicroseconds(StepDelay);
  }    //------------
  LeftFootPutDown();
}
void ShuffleStepRight() {
  for(int j = 0; j < 3; j++) {
    LeanRight();    // first movement from startup; not used in normal walking routine
    //------
    LeftFootStepForwardSecondHalf();  // begin walk forward loop
    LeftFootStepBackward();
    LeftFootPutDown();  //sensor looking right
    LeanLeft();
    RightFootStepBackward();
    RightFootStepForwardSecondHalf();
    RightFootPutDown();
  }
}
void ShuffleStepLeft() {
  for(int j = 0; j < 3; j++) {
    LeanLeft();    // first movement from startup; not used in normal walking routine
    //------
    RightFootStepForwardSecondHalf();  // begin walk forward loop
    RightFootStepBackward();
    RightFootPutDown();  //sensor looking right
    LeanRight();
    LeftFootStepBackward();
    LeftFootStepForwardSecondHalf();
    LeftFootPutDown();
  }
}
void LeanAndFootWiggle() {
  LeanRight();
  LeftAnkleWiggle();
  LeftFootPutDown();
  LeanLeft();
  RightAnkleWiggle();
  RightFootPutDown();
}
void LeanAndLegTwist() {
  LeanRight();    // first movement from startup; not used in normal walking routine
  LeftLegTwist();
  LeftFootPutDown();
  LeanLeft();
  RightLegTwist();
  RightFootPutDown();
}
void BobTheBoB() {
  for( i=0;i<=Duration;i++) {
    AnkleMove2(Right);
    AnkleMove2(Left);
    delayMicroseconds(StepDelay);
  }
  delay(200);
  for( i=0;i<=Duration;i++) {
    AnkleMove4(Right);
    AnkleMove4(Left);
    delayMicroseconds(StepDelay);
  }
  for( i=0;i<=Duration;i++) {
    AnkleMove1(Right);
    AnkleMove1(Left);
    delayMicroseconds(StepDelay);
  }
  delay(200);
  for( i=0;i<=Duration;i++) {
    AnkleMove3(Right);
    AnkleMove3(Left);
    delayMicroseconds(StepDelay);
  }
}

//                                                end Dance Moves
/*****************************************************************************************************
 * Individual Moves                                       *
 ******************************************************************************************************/

// each affects single servo position changes

void HipMove1 (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeInOut(i);
    Pos = RightHipMid + easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeInOut(i);
    Pos = LeftHipMid + easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove1i (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeIn(i);
    Pos = RightHipMid + easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeIn(i);
    Pos = LeftHipMid + easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove1o (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeOut(i);
    Pos = RightHipMid + easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeOut(i);
    Pos = LeftHipMid + easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove2 (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeInOut(i);
    Pos = RightHipRight - easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeInOut(i);
    Pos = LeftHipRight - easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove2i (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeIn(i);
    Pos = RightHipRight - easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeIn(i);
    Pos = LeftHipRight - easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove2o (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeOut(i);
    Pos = RightHipRight - easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeOut(i);
    Pos = LeftHipRight - easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove3 (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeInOut(i);
    Pos = RightHipMid - easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeInOut(i);
    Pos = LeftHipMid - easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove3i (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeIn(i);
    Pos = RightHipMid - easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeIn(i);
    Pos = LeftHipMid - easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove3o (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeOut(i);
    Pos = RightHipMid - easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeOut(i);
    Pos = LeftHipMid - easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove4 (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeInOut(i);
    Pos = RightHipLeft + easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeInOut(i);
    Pos = LeftHipLeft + easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove4i (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeIn(i);
    Pos = RightHipLeft + easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeIn(i);
    Pos = LeftHipLeft + easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void HipMove4o (int WhichServo) {
  Ease.setTotalChangeInPosition(HipRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeOut(i);
    Pos = RightHipLeft + easedPosition;
    RightHipServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeOut(i);
    Pos = LeftHipLeft + easedPosition;
    LeftHipServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove1 (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeInOut(i);
    Pos = RightAnkleMid + easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeInOut(i);
    Pos = LeftAnkleMid - easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove1i (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeIn(i);
    Pos = RightAnkleMid + easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeIn(i);
    Pos = LeftAnkleMid - easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove1o (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeOut(i);
    Pos = RightAnkleMid + easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeOut(i);
    Pos = LeftAnkleMid - easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove2 (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeInOut(i);
    Pos = RightAnkleMid - easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeInOut(i);
    Pos = LeftAnkleMid + easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove2i (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeIn(i);
    Pos = RightAnkleMid - easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeIn(i);
    Pos = LeftAnkleMid + easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove2o (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeOut(i);
    Pos = RightAnkleMid - easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeOut(i);
    Pos = LeftAnkleMid + easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove3 (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeInOut(i);
    Pos = RightAnkleDown - easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeInOut(i);
    Pos = LeftAnkleDown + easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove3i (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeIn(i);
    Pos = RightAnkleDown - easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeIn(i);
    Pos = LeftAnkleDown + easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove3o (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeOut(i);
    Pos = RightAnkleDown - easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeOut(i);
    Pos = LeftAnkleDown + easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove4 (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeInOut(i);
    Pos = RightAnkleUp + easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeInOut(i);
    Pos = LeftAnkleUp - easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove4i (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeIn(i);
    Pos = RightAnkleUp + easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeIn(i);
    Pos = LeftAnkleUp - easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
void AnkleMove4o (int WhichServo) {
  Ease.setTotalChangeInPosition(AnkleRange); 
  switch (WhichServo) {
  case 1:  //right
    easedPosition=Ease.easeOut(i);
    Pos = RightAnkleUp + easedPosition;
    RightAnkleServo.writeMicroseconds(Pos);
    break;
  case 2:  // left
    easedPosition=Ease.easeOut(i);
    Pos = LeftAnkleUp - easedPosition;
    LeftAnkleServo.writeMicroseconds(Pos);
    break;
  }
}
//                                                    end Individual Moves
/****************************************************************************************************************
 * other functions                                         *
 *****************************************************************************************************************/


void Listen() {
  int PinCheck;
  for(int x = 1; x < 6; x++) {
    PinCheck = analogRead(AudioPin);  // check clap detector
    if (PinCheck > 10) {  // clap detected
      AudioFlag = 1;
    }
  }
}
void HearingCheck() {
  if (AudioFlag > 0) {
    switch(Foot) {
    case 1:  // feet are reversed from typical reaction because value is from last call from TestFooting(Left)
      RightFootPutDown();
      break;
    case 2:
      LeftFootPutDown();
      break;
    }
    Celebrate();
    // add pause routine/object detection/alarmed LED call here: watch out for any Foot value changes!!!
    switch(Foot) {
    case 1:  // again, reactions reversed
      LeanLeft();
      break;
    case 2:
      LeanRight();
      break;
    }
    AudioFlag = 0;
  }
}
void TestFooting() {
  //delay(500);
  switch(Foot) {
  case 1:
    LightDetected = analogRead(RightFoot);
//    Serial.println(LightDetected);
//    Serial.println(" ");
    if(LightDetected < LightThreshold) {
      for(int n = 0; n < 2; n++) {  // step backwards 3
        LeanLeft();
        RightFootStepBackward();
        RightFootPutDown();
        LeanRight();
        LeftFootStepBackward();
        LeftFootPutDown();
      }
      LeanLeft();
      RightFootStepBackwardFirstHalf();        
      RightFootPutDown();
      for(int n = 0; n < 2; n++) {  // turn away from edge 4
        SpinLeft();
      }
      EdgeDetection = 1;  // must do extra hip and ankle moves to be in proper stance on return
      Navigate();  // check for obstacles
      LeanLeft();
      RightFootStepForwardSecondHalf();
      RightFootPutDown();  // return to different position from normal avoidance loop
      EdgeDetection = 0;

    }
    break;
  case 2:
    LightDetected = analogRead(LeftFoot);
//    Serial.println(LightDetected);
//    Serial.println(" ");
    if(LightDetected < LightThreshold) {
      for(int n = 0; n < 2; n++) {  // 3
        LeanRight();
        LeftFootStepBackward();
        LeftFootPutDown();
        LeanLeft();
        RightFootStepBackward();
        RightFootPutDown();
      }
      LeanRight();
      LeftFootStepBackwardFirstHalf();        
      LeftFootPutDown();
      for(int n = 0; n < 2; n++) {  // 4
        SpinRight();
      }
      EdgeDetection = 1;
      Navigate();        
      LeanRight();
      LeftFootStepForwardSecondHalf();
      LeftFootPutDown();
      EdgeDetection = 0;
    }
    break;

  }
}


void Celebrate() {
  SendI2C(3);  // start dance LED sequences
  RandomNumber = random(4);  // random number between 0 and 4-1
  switch (RandomNumber) {
  case 0:
    for(int i = 1; i < 3; i++) {
      ShuffleStep();
    }
    break;
  case 1:
    LeanAndFootWiggle();
    break;
  case 2:
    for(int i = 1; i < 3; i++) {
      LeanAndLegTwist();
    }
    break;
  case 3:
    for(int i = 1; i < 4; i++) {
      BobTheBoB();
    }
    break;
  }
  if(DanceTrigger == 1) {  // celebration called during fanmode
    SendI2C(6);  // start static LED sequences
  }
  else {  // celebration called during walking
    SendI2C(2);  // start walking LED sequence
  }
}
void Navigate() {
  GetDistance();
  if (Distance < DistanceMax && Distance > 0) {  
    switch (Foot) {  // shift to start standing position, aka position A
    case 1:  // right foot up, will turn left      
      SendI2C(4);  // start right directional avoidance LED sequences
      if(EdgeDetection < 1) {
        RightFootPutDown();
      }
      SpinLeft();
      GetDistance();
      GoNoGo();
      SpinLeft();
      SpinLeft();  // two more turns to help clear obstacle
      if(EdgeDetection < 1) {
        LeanLeft();     // restart with LeanLeft
      }
      break;

    case 2:
      SendI2C(5);  // start left directional avoidance LED sequences
      if(EdgeDetection < 1) {
        LeftFootPutDown();
      }
      SpinRight();
      GetDistance();
      GoNoGo();
      for(int n = 0; n < 2; n++) {
        SpinRight();  // two more turns to help clear obstacle
      }
      if(EdgeDetection < 1) {
        LeanRight();
      }
      break;
    }
  }
  SendI2C(2);  // walking LED sequence
}
void GetDistance() {
  Distance = sonar.ping(); // Send ping, get flight time in microseconds (uS).
}
void GoNoGo() {
  if (Distance < DistanceMax && Distance > 0) {
    do {
      switch (Foot) {
      case 1:
        SpinLeft();
        break;
      case 2:
        SpinRight();
        break;
      }        
      GetDistance();  // read distance again
    } 
    while (Distance < DistanceMax && Distance > 0);
  }   
}    
void Calibrate() {
  if(CalibrationCounter == 0) {  // gets baseline "dark" value from edge detector
    switch(Foot) {
    case 1:
      DarkCalibration = analogRead(RightFoot);
    Serial.println("RIGHT");
   Serial.println(DarkCalibration);
      break;
    case 2:
      DarkCalibration = analogRead(LeftFoot);
    Serial.println("LEFT");
    Serial.println(DarkCalibration);
      break;
    }
    CalibrationCounter = 1;  // set for value compare values step   
  }
  else {
    switch(Foot) {
      case 1:
        LightDetected = analogRead(RightFoot);
        break;
      case 2:
        LightDetected = analogRead(LeftFoot);
        break;
    }
    LightThreshold = LightDetected + (((DarkCalibration - LightDetected)/Denominator)*Numerator);  
    // due to slow CdS cell response time brighter conditions mean lower read values when foot is first placed down.
    // As opposed to waiting for the cell to settle (and potentially affect the gait) this calibration routine was implemented.
    CalibrationCounter = 0;  // resets counter
//    Serial.println(LightDetected);
//    Serial.println(LightThreshold);
  }
}
void SendI2C(byte CommandByte) {
  Wire.beginTransmission(42); // transmit to slave arduino @ address 42
  Wire.write(CommandByte);  // sends one byte, no return  
  Wire.endTransmission();    // stop transmitting
}

